const fs = require('fs');
const puppeteer = require('puppeteer');
const mkdirp = require('mkdirp');
const mktmpdir = require('mktmpdir');
const { execSync } = require('child_process');
var rimraf = require("rimraf");

const LANDING_PAGE_URL = "https://videodirect.amazon.com/home/sign-in?ref_=avd_hm_si";
const DASHBOARD_URL = "https://videocentral.amazon.com/home/dashboard";
const SEASON_PAGE_URL = "https://videocentral.amazon.com/content/edit/season?id=";
const USER_DATA_DIR = `/tmp/videodirectotron/`;
const WIDTH = 1280;
const HEIGHT = 720;
const FRAME_RATE = 3;

const VideoDirect = class VideoDirect {
  async browser() {
    if (this._browser) {
      return (this._browser);
    }

    this._browser = await puppeteer.launch({
      headless: false,
      args: [
        '--no-sandbox',
        '--disable-setuid-sandbox',
        '--disable-infobars',
        `--window-size=${WIDTH},${HEIGHT}`,
      ],
      defaultViewport: null,
      slowMo: 100,
      executablePath: process.env.PUPPETEER_EXECUTABLE_PATH || 'google-chrome-unstable',
      userDataDir: USER_DATA_DIR,
    });
    return (this._browser);
  }

  async page() {
    if (this._page) {
      return (this._page)
    }

    const browser = await this.browser();
    this._page = await browser.newPage();
    return (this._page)
  }

  async signIn() {
    const page = await this.page();

    await page.goto(LANDING_PAGE_URL);
    await page.waitFor(20000);

    return {};
  }

  async goToSeason(seasonId) {
    const page = await this.page();

    await page.goto(SEASON_PAGE_URL + seasonId);

    await page.waitForSelector("*[data-a-tab-name=episodes]");

    await page.click("*[data-a-tab-name=episodes]");

    await new Promise((resolve) => setTimeout(resolve, 10000));
  }

  async getEpisodeData(episodeMarkerNode, name) {
    const page = await this.page();

    let episodeDataNode = await episodeMarkerNode.$(`.dvp-episode-${name}`);

    return (await (await episodeDataNode.getProperty("value")).jsonValue());
  }

  async getEpisodeDetails(episodeMarkerNode) {
    const page = await this.page();
    const id = await this.getEpisodeData(episodeMarkerNode, 'id');

    let episodeHeaderBox = await episodeMarkerNode.$(".episode-btn-expand");

    await episodeHeaderBox.click();
    await page.waitForSelector(`#episode-page-form-${id} .dv-cooper-file-upload-filename-text`);

    const mezzanineFile = (await (await (await page.$(`#episode-page-form-${id} .dv-cooper-file-upload-filename-text`)).getProperty("innerText")).jsonValue()).replace(/^\s+|\s+$/g, '');
    const itemId = await page.evaluate((id) => {
      return document.querySelector(`#episode-page-form-${id} [name=itemId]`).value;
    }, id);

    await episodeHeaderBox.click();

    return ({ mezzanine_file: mezzanineFile, item_id: itemId });
  }

  async getAllEpisodeData(episodeMarkerNode) {
    return ({
      id: (await this.getEpisodeData(episodeMarkerNode, 'id')),
      number: parseInt(await this.getEpisodeData(episodeMarkerNode, 'sequence')),
      is_publishable: (await this.getEpisodeData(episodeMarkerNode, 'is-publishable')),
      has_valid_caption: (await this.getEpisodeData(episodeMarkerNode, 'has-valid-caption')),
      details: (await this.getEpisodeDetails(episodeMarkerNode))
    });
  }

  async findEpisode(seasonId, episodeNumber) {
    const page = await this.page();

    await this.goToSeason(seasonId);

    for (let episodeMarkerNode of (await page.$$(".dvp-episode-marker"))) {
      let episodeNumberNode = await episodeMarkerNode.$(".dvp-episode-sequence");
      if (parseInt(await this.getEpisodeData(episodeMarkerNode, 'sequence')) == parseInt(episodeNumber)) {
        return (await this.getAllEpisodeData(episodeMarkerNode));
      }
    }
  }

  async findAllEpisodesBySeason(seasonId) {
    const page = await this.page();

    await this.goToSeason(seasonId);
    let episodes = [];
    for (let episodeMarkerNode of (await page.$$(".dvp-episode-marker"))) {
      let episodeId = await this.getEpisodeData(episodeMarkerNode, 'id')
      episodes.push(await this.getAllEpisodeData(episodeMarkerNode));
    }
    return (episodes);
  }

  async findAllEpisodesBySeasons(seasonIds) {
    const page = await this.page();

    let episodesBySeason = {};
    for (let seasonId of seasonIds) {
      episodesBySeason[seasonId] = await this.findAllEpisodesBySeason(seasonId)
    }
    return (episodesBySeason);
  }

  async getMonthlyReport(year, month) {
    const page = await this.page();

    await page.goto(DASHBOARD_URL);
    await page.waitForSelector("#performanceMetricsExportDropdown .a-button-splitdropdown button");
    await page.click("#performanceMetricsExportDropdown .a-button-splitdropdown button");
    for (let monthReportLink of (await page.$$("a"))) {

      if ((await (await monthReportLink.getProperty("id")).jsonValue()).match(/^exportPerformanceMetrics_[0-9]+$/)) {
        let value = await page.evaluate((monthReportLink) => {
          return monthReportLink.getAttribute("data-value")
        }, monthReportLink)

        if (value.match(`${year}-${month}`)) {
          const tmpDir = await new Promise((resolve) => {
            mktmpdir((err, dir) => resolve(dir));
          })
          await page._client.send('Page.setDownloadBehavior', { behavior: 'allow', downloadPath: tmpDir });
          await page.evaluate((monthReportLink) => {
              monthReportLink.click();
          }, monthReportLink);
          await page.waitFor(10000);

          const csvFile = fs.readdirSync(tmpDir)[0];

          if (csvFile) {
            return ({ path: `${tmpDir}/${csvFile}` });
          }
        }
      }
    }
    return ({ error: `No report for ${year}/${month}`})
  }

  async getMonthlyFinancialReport(year, month) {
    const page = await this.page();

    await page.goto(DASHBOARD_URL);
    await page.waitForSelector("[name=estimatedRevenueDisplayPeriod]");
    await page.evaluate(() => {
      document.querySelector("[name=estimatedRevenueDisplayPeriod]").parentElement.querySelector("[role=button]").click()
    })

    for (let monthReportLink of (await page.$$(".a-popover.a-dropdown a"))) {

      let value = await page.evaluate((monthReportLink) => {
        return monthReportLink.getAttribute("data-value")
      }, monthReportLink)
      if (value.match(`${year}-${month <= 9 ? '0' + month : month}`)) {
        const tmpDir = await new Promise((resolve) => {
          mktmpdir((err, dir) => resolve(dir));
        })
        await page._client.send('Page.setDownloadBehavior', { behavior: 'allow', downloadPath: tmpDir });
        await page.evaluate((monthReportLink) => {
            monthReportLink.click();
            document.querySelector("[type=submit][value=exportEstimatedRevenue]").click();
        }, monthReportLink);
        await page.waitFor(10000);

        const csvFile = fs.readdirSync(tmpDir)[0];

        if (csvFile) {
          return ({ path: `${tmpDir}/${csvFile}`, estimate: !!csvFile.match(/estimate/i) });
        }
      }
    }
    return ({ error: `No report for ${year}/${month}`})
  }

  async createEpisode(seasonId, title, description, releaseDate, mezzPath, mezzLocale, captionsPath, captionsFramerate, captionsLocale, bannerPath) {
    const page = await this.page();

    await this.goToSeason(seasonId);

    await page.click("#item-save-add-episode-button-announce");
    await page.waitForSelector(".dvp-new-episode-box-group");

    const itemId = await page.evaluate(() => {
      return document.querySelector("[name=itemId]").value;
    });

    await page.type(".episode-page-form [name='localizedTitleNameList[0].text']", title);

    await page.type(".episode-page-form [name='localizedSynopsisList[0].text']", description);

    await page.evaluate((releaseDate) => {
      document.querySelector('.episode-page-form [name=originalReleaseDate]').value = new Date(releaseDate).getTime();
    }, releaseDate);

    /* mezzanineFile */
    await page.evaluate((mezzLocale) => {
      document.querySelector('.episode-page-form [name="mezz.locale"]').value = mezzLocale;
    }, mezzLocale);
    await (await page.$(".episode-page-form #file-data-Mezz")).uploadFile(mezzPath);

    if (captionsPath && captionsFramerate && captionsLocale) {
      /* captions */
      await page.click("[data-action='dvp-video-assets-add-caption'] button");
      await page.evaluate((captionsLocale) => {
        document.querySelector('.episode-page-form [name="timedText[0].locale"]').value = captionsLocale;
      }, captionsLocale);
      await page.evaluate((captionsFramerate) => {
        document.querySelector('.episode-page-form [name="timedText[0].captionFrameRateTimeCodeMethod"]').value = captionsFramerate;
      }, captionsFramerate);
      await (await page.$(".episode-page-form #file-data-Captions")).uploadFile(captionsPath);
    }

    if (bannerPath) {
      /* banner */
      await (await page.$(".episode-page-form #file-data-BoxArt4_3")).uploadFile(bannerPath);
    }

    await page.click("[data-action=dv-cooper-save-episode] input");
    await page.waitForSelector(".save-success-alert", { visible: true, timeout: 300000 });

    return ({ itemId: itemId })
  }

  async close() {
    const browser = await this.browser();
    await browser.close();
  }

  async startRecording() {
    if (this._recordingInterval) {
      return;
    }
    this._frameNum = 0;
    this._framesTmpDir = await new Promise((resolve) => {
      mktmpdir((err, dir) => resolve(dir));
    });
    this._framePromises = [];

    const page = await this.page();
    this._recordingInterval = setInterval(() => {
      try {
        const frame = `${this._framesTmpDir}/${process.pid}-frame${String(this._frameNum++).padStart(5, '0')}.png`;
        this._framePromises.push(this._page.screenshot({ path: frame }));
      } catch (e) {
        // ignore errors, they are most likely race conditions
      }
    }, 1000 / FRAME_RATE);
  }

  async stopRecording() {
    if (this._recordingInterval) {
      clearInterval(this._recordingInterval);
    }
    await Promise.all(this._framePromises);

    const output = `/tmp/videodirectotron-${process.pid}.mp4`;
    execSync(`ffmpeg -r ${FRAME_RATE} -f image2 -i ${this._framesTmpDir}/${process.pid}-frame%05d.png -vcodec libx264 -crf 25 -pix_fmt yuv420p ${output}`);
    rimraf.sync(this._framesTmpDir);

    return (output);
  }
}

module.exports = VideoDirect;
